@@ -328,7 +328,7 @@ impl IntoPy<PyObject> for PyEinsumArgs {
 impl Einsum {
     #[cfg(feature = "real-pyo3")]
     #[new]
-    #[args(args = "*", out_axes, name = "None")]
+    #[args(args = "*", out_axes, name)]
     fn py_new(
         args: Vec<(CircuitRc, EinsumAxes)>,
         out_axes: EinsumAxes,
@@ -340,7 +340,6 @@ impl Einsum {
     }
 
     #[staticmethod]
-    #[pyo3(name = "None")]
     pub fn from_einsum_string(
         string: String,
         nodes: Vec<CircuitRc>,
@@ -363,7 +362,6 @@ impl Einsum {
     }
 
     #[staticmethod]
-    #[pyo3(name = "None")]
     pub fn from_spec_py(
         spec: EinsumSpec,
         circuits: Vec<CircuitRc>,
