@@ -7,6 +7,7 @@ use super::{
     repr::RustRepr,
     HashBytes,
 };
+use base16::encode_lower;
 use num_bigint::BigUint;
 use pyo3::pyfunction;
 
@@ -98,7 +99,11 @@ pub fn oom_fmt<T: Into<BigUint>>(num: T) -> String {
 /// context maps children to variable names.
 /// we use a top sort to determine to order of calls to this function so that
 /// context always has the names for circuits we'll need to repr this one
-pub fn repr_circuit_one_level(circuit: &CircuitRc, context: &HashMap<CircuitRc, String>) -> String {
+pub fn repr_circuit_one_level(
+    circuit: &CircuitRc,
+    context: &HashMap<CircuitRc, String>,
+    tensors_as_randn: bool,
+) -> String {
     let node_string = |x: &CircuitRc| format!("{}.clone()", context[x]);
     match circuit.as_ref() {
         Circuit::Einsum(e) => {
@@ -115,23 +120,34 @@ pub fn repr_circuit_one_level(circuit: &CircuitRc, context: &HashMap<CircuitRc,
                 e.name_cloned().repr()
             )
         }
-        Circuit::ArrayConstant(a) => format!(
-            "ArrayConstant::randn_seeded({}, {}, {}, {}).rc()",
-            a.compute_shape().repr(),
-            a.name_cloned().repr(),
-            a.info().device_dtype.repr(),
-            a.value.hash_usize().unwrap(),
-        ),
+        Circuit::ArrayConstant(a) => {
+            if tensors_as_randn {
+                format!(
+                    "ArrayConstant::randn_seeded({}, {}, {}, {}).rc()",
+                    a.info().shape.repr(),
+                    a.name_cloned().repr(),
+                    a.info().device_dtype.repr(),
+                    a.value.hash_usize().unwrap(),
+                )
+            } else {
+                let hash_base16 = a.save_rrfs().unwrap();
+                format!(
+                    "ArrayConstant::from_hash({},{}).unwrap().rc()",
+                    a.name_cloned().repr(),
+                    (&hash_base16 as &str).repr(),
+                )
+            }
+        }
         Circuit::Symbol(s) => format!(
             "Symbol::nrc({}, {}, {})",
-            s.compute_shape().repr(),
+            s.info().shape.repr(),
             s.uuid.repr(),
             s.name_cloned().repr()
         ),
         Circuit::ScalarConstant(s) => format!(
             "ScalarConstant::nrc({}, {}, {})",
             s.value.repr(),
-            s.compute_shape().repr(),
+            s.info().shape.repr(),
             s.name_cloned().repr()
         ),
         Circuit::Add(a) => {
@@ -153,8 +169,11 @@ pub fn repr_circuit_one_level(circuit: &CircuitRc, context: &HashMap<CircuitRc,
         Circuit::Index(i) => format!(
             "Index::nrc({}, {}, {})",
             node_string(&i.node),
-            i.index
-                .repr(i.node.info().shape.clone(), &i.info().device_dtype),
+            i.index.repr(
+                i.node.info().shape.clone(),
+                &i.info().device_dtype,
+                tensors_as_randn
+            ),
             i.name_cloned().repr()
         ),
         Circuit::GeneralFunction(g) => {
@@ -167,7 +186,10 @@ pub fn repr_circuit_one_level(circuit: &CircuitRc, context: &HashMap<CircuitRc,
                     g.name_cloned().repr()
                 )
             } else {
-                panic!("cant print non-official generalfunctions");
+                panic!(
+                    "cant print non-official generalfunctions {} {:?}",
+                    g.spec.name, g.spec.function
+                );
             }
         }
         Circuit::Concat(c) => {
@@ -184,8 +206,12 @@ pub fn repr_circuit_one_level(circuit: &CircuitRc, context: &HashMap<CircuitRc,
                 "Scatter::nrc({}, {}, {}, {})",
                 node_string(&s.node),
                 // indices are into the output zero-padded array, so we use the shape of s instead of s.node
-                s.index.repr(s.info().shape.clone(), &s.info().device_dtype),
-                s.compute_shape().repr(),
+                s.index.repr(
+                    s.info().shape.clone(),
+                    &s.info().device_dtype,
+                    tensors_as_randn
+                ),
+                s.info().shape.repr(),
                 s.name_cloned().repr()
             )
         }
@@ -221,8 +247,8 @@ pub fn repr_circuit_one_level(circuit: &CircuitRc, context: &HashMap<CircuitRc,
 ///     )
 /// };
 /// ```
-#[pyfunction]
-pub fn rust_expression_notation_circuit(circuit: CircuitRc) -> String {
+#[pyfunction(tensors_as_randn = "false")]
+pub fn rust_expression_notation_circuit(circuit: CircuitRc, tensors_as_randn: bool) -> String {
     // Possible improvements:
     // * better variable names (if the node has a name that is a valid Rust identifier, doesn't shadow anything, ..., use that)
     // * inline single use variables?
@@ -242,7 +268,7 @@ pub fn rust_expression_notation_circuit(circuit: CircuitRc) -> String {
         let key: CircuitRc = CircuitRc(node.0.clone());
         context.insert(key.clone(), format!("node{}", i));
 
-        let node_source = repr_circuit_one_level(node, &context);
+        let node_source = repr_circuit_one_level(node, &context, tensors_as_randn);
         if i < order.len() - 1 {
             source.push_str(format!("    let {} = {};\n", context[node], node_source).as_str());
         } else {
